@@ -6,7 +6,7 @@ use lib 'lib';
 
 package File::Util;
 {
-  $File::Util::VERSION = '4.130460'; # TRIAL
+  $File::Util::VERSION = '4.130483'; # TRIAL
 }
 
 use File::Util::Definitions qw( :all );
@@ -49,6 +49,8 @@ sub new {
 
    $this->{opts} = $in || { };
 
+   $this->{opts}->{onfail} ||= 'die';
+
    # let constructor argument override globals, but set
    # constructor opts to global values if they have not
    # overridden them...
@@ -140,7 +142,7 @@ sub list_dir {
 
 # INPUT VALIDATION
 
-   return $this->_throw( 'no such file' => { filename => $dir } )
+   return $this->_throw( 'no such file' => { opts => $opts, filename => $dir } )
       unless -e $dir;
 
    # whack off any trailing directory separator, except for root directories
@@ -259,7 +261,7 @@ sub list_dir {
 
    if ( my $cb = $opts->{callback} ) {
 
-      $this->throw( qq(callback "$cb" not a coderef) )
+      $this->throw( qq(callback "$cb" not a coderef), $opts )
          unless ref $cb eq 'CODE';
 
       $cb->( $dir, \@dirs, \@items, scalar split_path( $dir ) - 1 );
@@ -267,7 +269,7 @@ sub list_dir {
 
    if ( my $cb = $opts->{d_callback} ) {
 
-      $this->throw( qq(d_callback "$cb" not a coderef) )
+      $this->throw( qq(d_callback "$cb" not a coderef), $opts )
          unless ref $cb eq 'CODE';
 
       $cb->( $dir, \@dirs, scalar split_path( $dir ) - 1 );
@@ -275,7 +277,7 @@ sub list_dir {
 
    if ( my $cb = $opts->{f_callback} ) {
 
-      $this->throw( qq(f_callback "$cb" not a coderef) )
+      $this->throw( qq(f_callback "$cb" not a coderef), $opts )
          unless ref $cb eq 'CODE';
 
       $cb->( $dir, \@items, scalar split_path( $dir ) - 1 );
@@ -730,7 +732,7 @@ sub _match_or {
 sub _as_tree {
    my $this = shift @_;
    my $opts = $this->_remove_opts( \@_ );
-   my $dir  = shift @_ || '.';
+   my $dir  = shift @_;
    my $tree = {};
 
    my $treeify = sub
@@ -894,8 +896,9 @@ sub load_file {
                ? $READ_LIMIT
                : 0;
 
-   return $this->_throw ( 'bad read_limit' => { bad => $read_limit } )
-      if $read_limit =~ /\D/;
+   return $this->_throw(
+      'bad read_limit' => { opts => $in, bad => $read_limit }
+   ) if $read_limit =~ /\D/;
 
    # support old-school "FH" option, *and* the new, more sensible "file_handle"
    $in->{FH} = $in->{file_handle} if defined $in->{file_handle};
@@ -1090,7 +1093,7 @@ sub load_file {
             }
          );
 
-      $this->_seize( $clean_name, $fh );
+      $this->_seize( $clean_name, $fh, $in );
    }
 
    # call binmode on binary files for portability accross platforms such
@@ -1121,7 +1124,7 @@ sub load_file {
    }
    else {
       # release shadow-ed locks on the file
-      $this->_release( $fh );
+      $this->_release( $fh, $in );
 
       close $fh or return $this->_throw(
          'bad close',
@@ -1401,7 +1404,7 @@ sub write_file {
             );
 
          # lock file before I/O on platforms that support it
-         my $lockstat = $this->_seize( $clean_name, $write_fh );
+         my $lockstat = $this->_seize( $clean_name, $write_fh, $in );
 
          return unless $lockstat;
 
@@ -1437,7 +1440,7 @@ sub write_file {
          );
 
          # lock file before I/O on platforms that support it
-         my $lockstat = $this->_seize( $clean_name, $write_fh );
+         my $lockstat = $this->_seize( $clean_name, $write_fh, $in );
 
          return unless $lockstat;
       }
@@ -1461,7 +1464,7 @@ sub write_file {
 
    # release lock on the file
 
-   $this->_release( $write_fh ) unless $$in{no_lock} || !$USE_FLOCK;
+   $this->_release( $write_fh, $in ) unless $$in{no_lock} || !$USE_FLOCK;
 
    close $write_fh or
       return $this->_throw(
@@ -1481,13 +1484,15 @@ sub write_file {
 # File::Util::_seize()
 # --------------------------------------------------------
 sub _seize {
-   my ( $this, $file, $fh ) = @_;
+   my ( $this, $file, $fh, $opts ) = @_;
 
-   return $this->_throw( 'no handle passed to _seize.' ) unless $fh;
+   return $this->_throw( 'no handle passed to _seize.' => $opts )
+      unless $fh;
 
    $file = defined $file ? $file : ''; # yes, even files named "0" are allowed
 
-   return $this->_throw( 'no file name passed to _seize.' ) unless length $file;
+   return $this->_throw( 'no file name passed to _seize.' => $opts )
+      unless length $file;
 
    # forget seizing if system can't flock
    return $fh if !$CAN_FLOCK;
@@ -1498,7 +1503,7 @@ sub _seize {
 
    while ( @policy ) {
 
-      my $fh = &{ $_LOCKS->{ shift @policy } }( $this, $file, $fh );
+      my $fh = &{ $_LOCKS->{ shift @policy } }( $this, $file, $fh, $opts );
 
       return $fh if $fh || !scalar @policy;
    }
@@ -1512,9 +1517,10 @@ sub _seize {
 # --------------------------------------------------------
 sub _release {
 
-   my ( $this, $fh ) = @_;
+   my ( $this, $fh, $opts ) = @_;
 
-   return $this->_throw( 'not a filehandle.', { argtype => ref $fh } )
+   return $this->_throw(
+      'not a filehandle.' => { opts => $opts, argtype => ref $fh } )
       unless $fh && ref $fh eq 'GLOB';
 
    if ( $CAN_FLOCK ) { flock $fh, &Fcntl::LOCK_UN }
@@ -1679,7 +1685,8 @@ sub existent { my $f = _myargs( @_ ); defined $f ? -e $f : undef }
 # --------------------------------------------------------
 sub touch {
    my $this = shift @_;
-   my $file = shift( @_ ) || '';
+   my $file = shift @_ || '';
+   my $opts = $this->_remove_opts( \@_ );
    my $path;
 
    return $this->_throw(
@@ -1687,7 +1694,7 @@ sub touch {
       {
          meth    => 'touch',
          missing => 'a file name or file handle reference',
-         opts    => { },
+         opts    => $opts,
       }
    ) unless defined $file && length $file;
 
@@ -1699,7 +1706,7 @@ sub touch {
       {
          filename => $file,
          dirname  => $path || '',
-         opts     => { },
+         opts     => $opts,
       }
    ) if -e $file && -d $file;
 
@@ -1713,7 +1720,7 @@ sub touch {
       {
          filename => $file,
          dirname  => $path,
-         opts     => { },
+         opts     => $opts,
       }
    ) if ( -e $path && !-r $path );
 
@@ -2047,7 +2054,7 @@ sub make_dir {
                exception => $!,
                dirname   => $dir,
                bitmask   => $bitmask,
-               opts     => $opts,
+               opts      => $opts,
             }
          );
    }
@@ -2065,7 +2072,8 @@ sub max_dives {
 
    if ( defined $arg ) {
 
-      return File::Util->new->_throw('bad max_dives') if $arg =~ /\D/;
+      return File::Util->new->_throw( 'bad max_dives' => { bad => $arg } )
+         if $arg =~ /\D/;
 
       $MAX_DIVES = $arg;
 
@@ -2076,6 +2084,19 @@ sub max_dives {
    return $MAX_DIVES;
 }
 
+# --------------------------------------------------------
+# File::Util::onfail()
+# --------------------------------------------------------
+sub onfail {
+   my ( $this, $arg ) = @_;
+
+   return unless blessed $this;
+
+   $this->{opts}->{onfail} = $arg if $arg;
+
+   return $this->{opts}->{onfail};
+}
+
 
 # --------------------------------------------------------
 # File::Util::read_limit()
@@ -2108,9 +2129,9 @@ sub diagnostic {
 
    if ( defined $arg ) {
 
-      $WANT_DIAGNOSTICS = !!$arg;
+      $WANT_DIAGNOSTICS = $arg ? 1 : 0;
 
-      $this->{opts}->{diag} = !!$arg
+      $this->{opts}->{diag} = $arg ? 1 : 0
          if blessed $this && $this->{opts};
    }
 
@@ -2453,9 +2474,9 @@ sub open_handle {
                );
 
             # lock file before I/O on platforms that support it
-            my $lockstat = $this->_seize( $clean_name, $fh );
+            my $lockstat = $this->_seize( $clean_name, $fh, $in );
 
-            return $lockstat unless $lockstat;
+            warn "returning $lockstat" && return $lockstat unless fileno $lockstat;
 
             if ( $mode ne 'read' ) {
 
@@ -2486,7 +2507,7 @@ sub open_handle {
                );
 
             # lock file before I/O on platforms that support it
-            my $lockstat = $this->_seize( $clean_name, $fh );
+            my $lockstat = $this->_seize( $clean_name, $fh, $in );
 
             return $lockstat unless $lockstat;
          }
@@ -2508,7 +2529,7 @@ sub open_handle {
                );
 
             # lock file before I/O on platforms that support it
-            my $lockstat = $this->_seize( $clean_name, $fh );
+            my $lockstat = $this->_seize( $clean_name, $fh, $in );
 
             return $lockstat unless $lockstat;
 
@@ -2542,7 +2563,7 @@ sub open_handle {
             );
 
             # lock file before I/O on platforms that support it
-            my $lockstat = $this->_seize( $clean_name, $fh );
+            my $lockstat = $this->_seize( $clean_name, $fh, $in );
 
             return $lockstat unless $lockstat;
          }
@@ -2563,16 +2584,18 @@ sub open_handle {
 sub unlock_open_handle {
    my( $this, $fh ) = @_;
 
-   return 1 if !$USE_FLOCK;
+   return 1 unless $USE_FLOCK;
 
-   my $ref_type = ref \$fh || '';
-
-   return $this->_throw( 'not a filehandle' => { argtype => $ref_type } )
-      unless $fh && $ref_type eq 'GLOB';
+   return $this->_throw(
+      'not a filehandle' => {
+         opts    => $this->_remove_opts( \@_ ),
+         argtype => ref $fh,
+      }
+   ) unless $fh && fileno $fh;
 
    return flock( $fh, &Fcntl::LOCK_UN ) if $CAN_FLOCK;
 
-   return 1;
+   return 0;
 }
 
 
@@ -2694,6 +2717,8 @@ sub AUTOLOAD {
 
       goto \&$name;
    }
+
+   die qq(Unknown method: File::Util::$name\n);
 }
 
 
@@ -2715,7 +2740,7 @@ File::Util - Easy, versatile, portable file handling
 
 =head1 VERSION
 
-version 4.130460
+version 4.130483
 
 =head1 DESCRIPTION
 
@@ -2737,7 +2762,46 @@ I<(See L<DOCUMENTATION|/DOCUMENTATION> section below.)>
 
 =head1 SYNOPSIS
 
-   # .......... GETTING STARTED ...........................................
+   # use File::Util in your program
+   use File::Util;
+
+   # create a new File::Util object
+   my $f = File::Util->new();
+
+   # load a file into a variable
+   my $content = $f->load_file( 'some_file.txt' );
+
+   # write content to a file
+   $f->write_file( 'some_file.txt' => $content );
+
+=head1 DOCUMENTATION
+
+You can do much more with File::Util than just the basic examples this
+document.  For an explanation of all the features available to you,
+take a look at these other reference materials:
+
+=over
+
+=item B<The Manual>
+
+The L<File::Util::Manual> is the complete reference document explaing every
+available feature and object method.
+
+=item B<The "Nutshell">
+
+The L<File::Util::Manual::Examples> document has a long list of small, reusable
+code snippets and techniques to use in your own programs.
+
+=item B<The Cookbook>
+
+The L<File::Util::Cookbook> contains examples of complete, working programs
+that use File::Util to easily accomplish tasks which require file handling.
+
+=back
+
+=head1 BASIC USAGE
+
+=head2 Getting Started
 
    # use File::Util in your program
    use File::Util;
@@ -2752,7 +2816,7 @@ I<(See L<DOCUMENTATION|/DOCUMENTATION> section below.)>
    # ...you can enable diagnostics for individual objects:
    $f = File::Util->new( diag => 1 );
 
-   # .......... FILE OPERATIONS ...........................................
+=head2 File Operations
 
    # load content into a variable, be it text, or binary, either works
    my $content = $f->load_file( 'Meeting Notes.txt' );
@@ -2770,7 +2834,7 @@ I<(See L<DOCUMENTATION|/DOCUMENTATION> section below.)>
    );
 
    # try binary this time
-   my $binary_content = $f->load_file( 'cat-movie.avi' );
+   my $binary_content = $f->load_file( 'barking-cat.avi' );
 
    # get some image data from somewhere...
    my $picture_data = get_image_upload();
@@ -2805,7 +2869,7 @@ I<(See L<DOCUMENTATION|/DOCUMENTATION> section below.)>
    # get the number of lines in a file
    my $log_line_count = $f->line_count( '/var/log/messages' );
 
-   # .......... FILE HANDLES ..............................................
+=head2 File Handles
 
    # get an open file handle for reading
    my $fh = $f->open_handle( file => 'Ian likes cats.txt', mode => 'read' );
@@ -2825,9 +2889,9 @@ I<(See L<DOCUMENTATION|/DOCUMENTATION> section below.)>
 
    print $fh 'Shout out to Bob!';
 
-   close $fh or die $!; # _never_ forget to close ;-)
+   close $fh or die $!; # don't forget to close ;-)
 
-   # .......... DIRECTORIES ...............................................
+=head2 Directories
 
    # get a listing of files, recursively, skipping directories
    my @files = $f->list_dir( '/var/tmp' => { files_only => 1, recurse => 1 } );
@@ -2863,7 +2927,7 @@ I<(See L<DOCUMENTATION|/DOCUMENTATION> section below.)>
    # get an entire directory tree as a hierarchal datastructure reference
    my $tree = $f->list_dir( '/my/podcasts' => { as_tree => 1 } );
 
-   # .......... GETTING FILE DETAILS ......................................
+=head2 Getting Information About Files
 
    print "My file has a bitmask of " . $f->bitmask( 'my.file' );
 
@@ -2874,26 +2938,9 @@ I<(See L<DOCUMENTATION|/DOCUMENTATION> section below.)>
    print 'My file was last modified on ' .
       scalar localtime $f->last_modified( 'my.file' );
 
-...There's B<_lots_> more.  See the L<File::Util::Manual> for more details and
-more features like advanced pattern matching in directories, directory walking,
-user-definable error handlers, and more.
-
-=head1 INSTALLATION
-
-To install this module type the following at the command prompt:
-
-   perl Build.PL
-   perl Build
-   perl Build test
-   sudo perl Build install
-
-On Windows systems, the "sudo" part of the command may be omitted, but you
-will need to run the rest of the install command with Administrative privileges
-
-=head1 DOCUMENTATION
-
-There's more than just this document!  Take a look at the L<File::Util::Manual>,
-the L<File::Util::Manual::Examples>, and the L<File::Util::Cookbook>.
+...See the L<File::Util::Manual> for more details and features like advanced
+pattern matching in directories, directory walking, user-definable error
+handlers, and more.
 
 =head1 METHODS
 
@@ -2955,6 +3002,8 @@ this document in a text terminal, open perldoc to the C<File::Util::Manual>.
 
 =item new                  I<(see L<new|File::Util::Manual/new>)>
 
+=item onfail               I<(see L<onfail|File::Util::Manual/onfail>)>
+
 =item open_handle          I<(see L<open_handle|File::Util::Manual/open_handle>)>
 
 =item read_limit           I<(see L<read_limit|File::Util::Manual/read_limit>)>
@@ -3115,18 +3164,22 @@ unicode support
 
 =back
 
-=head1 EXAMPLES
+=head1 INSTALLATION
 
-See L<File::Util::Manual>
+To install this module type the following at the command prompt:
 
-=head1 EXAMPLES (Full Programs)
+   perl Build.PL
+   perl Build
+   perl Build test
+   sudo perl Build install
 
-See L<File::Util::Cookbook>
+On Windows systems, the "sudo" part of the command may be omitted, but you
+will need to run the rest of the install command with Administrative privileges
 
 =head1 BUGS
 
 Send bug reports and patches to the CPAN Bug Tracker for File::Util at
-L<https://rt.cpan.org/Dist/Display.html?Name=File%3A%3AUtil>
+L<rt.cpan.org|https://rt.cpan.org/Dist/Display.html?Name=File%3A%3AUtil>
 
 =head1 SUPPORT
 
@@ -3169,6 +3222,8 @@ This software is distributed in the hope that it will be useful, but without
 any warranty; without even the implied warranty of merchantability or fitness
 for a particular purpose.
 
+This disclaimer applies to every part of the File::Util distribution.
+
 =head1 SEE ALSO
 
 The rest of the documentation:
