@@ -2,10 +2,11 @@ use strict;
 use warnings;
 
 use lib 'lib';
+use Scalar::Util qw( blessed );
 
 package File::Util::ProxyLog;
 {
-  $File::Util::ProxyLog::VERSION = '4.130460'; # TRIAL
+  $File::Util::ProxyLog::VERSION = '4.130483'; # TRIAL
 }
 
 use Time::HiRes;
@@ -14,6 +15,7 @@ use Data::Dumper;
    $Data::Dumper::Indent   = 2;
    $Data::Dumper::Terse    = 1;
    $Data::Dumper::Sortkeys = 1;
+   $Data::Dumper::Deparse  = 1;
 
 our $LOGFILE;
 our $LOGFH;
@@ -32,21 +34,42 @@ sub new
 sub AUTOLOAD
 {
    my ( $name ) = our $AUTOLOAD =~ /.*::(\w+)$/;
+   my $self     = shift;
 
-   my $self = shift @_;
-   my $time = time;
-   my $dump = qq(@{[ Dumper \@_ ]});
+   ## no critic
 
-   print $LOGFH <<__CALL__;
---------------------------------------------------------------------------------
+   no warnings 'redefine';
+   no strict 'refs';
+
+   ## use critic
+
+   my $method = \&{ qq(File::Util::$name) };
+
+   *{ qq(File::Util::$name) } = sub
+   {
+      my $self  = shift;
+      my $time  = time;
+      my $dump  = qq(@{[ Dumper \@_ ]});
+      my $delim = '-' x 80;
+
+      print $LOGFH <<__CALL__;
+$delim
 $time $name called with these args:
 $time   @{[ join "\n$time   ", split /\n/, $dump ]}
 __CALL__
 
+      unshift @_, $self if blessed $self;
+
+      goto $method;
+   };
+
    return $$self->$name( @_ );
+
+   # the below is cool, but doesn't work well, which is why it's commented-out
+   # unshift @_, $self and goto \&{ qq(File::Util::$name) };
 }
 
-sub DESTROY { close $LOGFH }
+sub DESTROY { if ( $LOGFH ) { close $LOGFH or die $! } return }
 
 1;
 
@@ -60,7 +83,7 @@ File::Util::ProxyLog - a call logging proxy class for File::Util
 
 =head1 VERSION
 
-version 4.130460
+version 4.130483
 
 =head1 DESCRIPTION
 
